#include <iostream>
#include<fstream>
#include<cstdlib>

#include <TCanvas.h>
#include <TROOT.h>
#include <TApplication.h>
#include <TFile.h>
#include <TH1F.h>
#include <omp.h>

#include "Garfield/ViewSignal.hh"
#include "Garfield/ViewDrift.hh"
#include "Garfield/ViewCell.hh"
#include "Garfield/ViewMedium.hh"
#include "Garfield/ViewField.hh"

#include "Garfield/TrackSrim.hh"
#include "Garfield/ComponentAnalyticField.hh"
#include "Garfield/ComponentConstant.hh"
#include "Garfield/MediumMagboltz.hh"
#include "Garfield/DriftLineRKF.hh"
#include "Garfield/Sensor.hh"
#include "Garfield/AvalancheMC.hh"
#include "Garfield/TrackHeed.hh"
#include "Garfield/Random.hh"
#include "Garfield/Plotting.hh"

using namespace Garfield;
using namespace ROOT;
using namespace std;



//电流前端电子学卷积函数
double Transferfunction(double t){     
  constexpr double risistance= 100 ;   //电阻设置
  constexpr double capacitance=  100;    //电容设置
  constexpr double tau=risistance*capacitance;
  return exp(t/tau);
}

void printsignaltotxt(Sensor& sensor,const std::string label,const int nTimebins,const double tstep,const double tmin,float name[]){     
  char title[60];
  sprintf(title,"currentsignal Ar_%.1f_CO2_%.1f_300K_%.2fAtm_%.0fV.txt",name[0],name[1],name[2],name[3]); 
  std::ofstream outfile;
  outfile.open(title,std::ios::out);
  outfile << "time     "<< "total current     "<< "electron current     " << "ion current\n";
  for(unsigned int i=0;i<nTimebins;++i){
    const double t=tstep/2+tmin+i*tstep;
    const double f=sensor.GetSignal(label,i);
    const double fe=sensor.GetElectronSignal(label,i);
    const double fion=sensor.GetIonSignal(label,i);
    outfile << t << "      "  << f << "      "  <<fe << "      "   << fion << "\n";
  }
}
  
int main(int argc, char * argv[]) {

	TApplication app("app", &argc, argv);
	MediumMagboltz gas;
	//gas.SetComposition("ar", 85., "co2", 15.);
	//gas.SetPressure(0.26 * AtmosphericPressure );
	//gas.SetTemperature(300);
  //gas.EnableAutoEnergyLimit(false);
  gas.SetMaxElectronEnergy(200);
	
	gas.LoadGasFile("ar_85_co2_15.gas");
	const std::string path = std::getenv("GARFIELD_HOME");
	gas.LoadIonMobility(path + "/Data/IonMobility_Ar+_Ar.txt");


	ComponentAnalyticField cmp;
	cmp.SetMedium(&gas);
	
	//
	const double rWire = 30.e-4;//cm
	const double rTube = 3.5;//cm
	//
	const double vWire = 1100.;
	const double vTube = 0.;
	//
	cmp.AddWire(0 , 0, 2* rWire, vWire, "s");
	cmp.AddTube(rTube, vTube, 0 , "t");
	cmp.AddReadout("s");

	//
	Sensor sensor;
	sensor.AddComponent(&cmp);
	sensor.AddElectrode(&cmp, "s");
 /*
  const double tstep = 0.5;
  const double tmin = -0.5 * tstep;
  const unsigned int nbins = 1000;
  sensor.SetTimeWindow(tmin, tstep, nbins);
  */
  
	// Set the signal time window.
 
  const double tstep = 0.1;     
  const double tmin = 0;     
  const double tmax=20000;             
  const unsigned int nbins = (tmax-tmin)/tstep;   
  sensor.SetTimeWindow(tmin, tstep, nbins);
  //sensor.SetTransferFunction(Transferfunction);
  sensor.ClearSignal();
 


	
	//----alpha in -----
	// Read the SRIM output file. 
	TrackSrim track(&sensor);
 	const std::string file = "Li_in_Ar_CO2_gas.txt";
 	if (!track.ReadFile(file)) {
 	std::cerr << "Reading Li_in_Ar_CO2_gas.txt file failed.\n";
 	return 1;
 	}
  	
	// Set the initial kinetic energy of the particle (in eV).
 	 track.SetKineticEnergy(0.84e6);
   track.SetWorkFunction(27.7);      
   track.SetFanoFactor(0.172);      
   const double za = 0.85 * (18. / 39.948) +0.15 * (22. / 44.0095);  // Set A and Z of the gas     (not sure what's the correct mixing law).
   track.SetAtomicMassNumbers(22. / za, 22);
  	// Specify how many electrons we want to be grouped to a cluster.
   //track.SetTargetClusterSize(100);
   track.EnableTransverseStraggling(false);
   track.EnableLongitudinalStraggling(false);
  
  DriftLineRKF drift(&sensor);
  drift.SetGainFluctuationsPolya(0., -1);
  drift.EnableIonTail();
  drift.EnableSignalCalculation();
  
  
  
  
	
  
  const double rTrack =3.0;     
  const double projectilex= rTrack;
  const double projectiley= sqrt(rTube * rTube - rTrack * rTrack);
  const double projectilez= 0.;  
  
  const double projectiledx=0.;    
  const double projectiledy=-1.; 
  const double projectiledz=0.; 
  
  const double projectileT=0.;   //**********
  const unsigned int nTracks = 1;  

  tr.NewTrack(projectilex, projectiley, projectilez, projectileT, projectiledx, projectiledy, projectiledz);

  while (track.GetCluster(xc, yc, zc, tc, nce, eDepc, extra)) {

    drift.SetIonSignalScalingFactor(nce);
    drift.DriftIon(xc, yc, zc, tc);
    drift.SetElectronSignalScalingFactor(nce);
    drift.DriftElectron(xc, yc, zc, tc);
}

  //printsignaltotxt(sensor,"s",nbins,tstep,tmin,name);  
  
  ViewSignal signalView;
  signalView.SetSensor(&sensor);
  signalView.PlotSignal("s");
  
  app.Run();
  return 0;
}
